<?php
namespace Estoque\Logic\GeralLancamentoEstoque;

use \Estoque\Entities\Custocmpprod;
use \Estoque\Entities\Itlanctoest;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Movilote;
use \Estoque\Entities\Posestdia;
use \Estoque\Util\Parametros;

use \Estoque\Logic\AjusteEstoque\Ajuste;
use \Estoque\Logic\AtualizacaoEstoque\CalculaCustoLiquido;
use \Estoque\Logic\Exception\ExceptionLancamentoEstoque;
use \Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
use \Estoque\Logic\ImplantacaoEstoque\Implantacao;

use \Geral\Entities\Almoxarifado;
use \Geral\Entities\Filial;
use \Geral\Entities\Localestoque;
use \Geral\Entities\Lote;
use \Geral\Entities\Prodfili;
use \Geral\Entities\Produto;
use \Geral\Entities\Retirada;

use \Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Geral\Util\NovoCodigo;

use \Planejamento\Entities\Cardadia;

use Setforms2\Runtime\Helper\Procedure\Param;

/**
 * Created by JetBrains PhpStorm.
 * User: marcelosilva
 * Date: 23/11/12
 * Time: 18:16
 * To change this template use File | Settings | File Templates.
 */
class LancamentoEstoque
{


    const ENTRADA = 'entrada';
    const SAIDA = 'saida';
    const AJUSTE = Lanctoestoq::LANCAMENTO_AJUSTE_DE_INVENTARIO;
    const IMPLANTACAO = Lanctoestoq::LANCAMENTO_IMPLANTACAO_DE_ESTOQUE;
    const CONTADOR_LANCTOESTOQ = 'LANCTOEST';


    public static function validaAlmoxLocalLoteSublote(&$almox, &$local, &$lote, &$subLote){
        if ($almox === Itlanctoest::SUBSTITUICAO_ESPACO) {
            $almox = Parametros::getAlmoxarifado();
        }

        if ($local === Itlanctoest::SUBSTITUICAO_ESPACO) {
            $local = Parametros::getLocalEstoque();
        }

        if ($lote === Itlanctoest::SUBSTITUICAO_ESPACO) {
            $lote = Parametros::getNumeroLote();
        }

        if ($subLote === Itlanctoest::SUBSTITUICAO_ESPACO) {
            $subLote = Parametros::getNumeroSubLote();
        }
    }


    public static function novoCodigo($nrorg, $cdfilial) {
        $novocodigo = Novocodigo::geraCodigo($nrorg, self::CONTADOR_LANCTOESTOQ.$cdfilial);
        return $novocodigo;
    }


    /**
     * Salva/altera itens.
     *
     * @param $nrorg
     * @param $filial
     * @param $nrLancto
     * @param $dtLancto
     * @param $seq
     * @param $cdProduto
     * @param $cdProdMovi
     * @param $qtProduto
     * @param $ordem
     * @param $vrBruto
     * @param $vrLiq
     * @param $lote
     * @param $subLote
     * @param $local
     * @param $almox
     * @param $vrConv
     * @param $tipoLancto
     * @param $operAtual
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function salvaItens(
        $nrorg, $filial, $nrLancto, $dtLancto, $seq, $cdProduto, $qtProduto, $vrTotal,
        $ordem, $vrBruto, $vrLiq, $lote, $subLote, $local, $almox, $qtEstoq, $vrEst, $tipoLancto, $operAtual
    ){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            self::produtoCadastrado($cdProduto, $nrLancto, $filial, $nrorg);

            self::insereItemLancamento(
                $filial, $nrorg, $cdProduto, $qtProduto, $vrTotal, $nrLancto, $dtLancto,
                $ordem, $seq, $vrBruto, $vrLiq, $lote, $subLote, $local, $almox,
                Itlanctoest::ESTOQUE_NAO_EVOLUIDO, $tipoLancto,
                $qtEstoq, $vrEst, $operAtual
            );

            $connection->flush();
            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    public static function atualizaItens(
        $nrorg, $filial, $nrLancto, $dtLancto, $seq, $cdProduto, $cdProdMovi, $qtProduto, $vrTotal,
        $idMedia, $ordem, $vrBruto, $vrLiq, $lote, $subLote, $local, $almox, $qtEstoq, $vrEstoq, $tipoLancto, $operAtual
    ){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            /** @var $objItem Itlanctoest */
            $objItem = Itlanctoest::consultaItem($filial, $nrLancto, $nrorg, $seq);

            $objItem->atualizaDadosProduto(
                $filial, $cdProduto, $qtProduto, $vrTotal, $vrLiq, $vrBruto, $qtEstoq, $vrEstoq, $nrorg, $operAtual
            );

            $connection->flush();
            $connection->commit();

            if ($tipoLancto === Itlanctoest::TIPO_AJUSTE) {
                Ajuste::validaOrdemDiferente($seq, $ordem);
                Ajuste::executaRetiradas(
                    $nrLancto, $dtLancto, $cdProduto, $cdProdMovi, $qtProduto, $idMedia, $seq,
                    $lote, $subLote, $local, $almox, $qtEstoq, $filial, $nrorg, $operAtual
                );
            }
        } catch (ExceptionLancamentoEstoque $e){
            throw $e;
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Atualiza lancamento.
     *
     * @param $nrorg
     * @param $filial
     * @param $nrLancto
     * @param $dtLancto
     * @param $dsLancto
     * @param $operAtual
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function atualizaLancamento($nrorg, $filial, $nrLancto, $dtLancto, $dsLancto, $operAtual){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            /** @var $lancto Lanctoestoq */
            $lancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);
            $lancto->atualizaLancamento($dsLancto, $dtLancto, $operAtual, $nrorg);

            $connection->flush();
            $connection->commit();

            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Exclui itens e atualiza o estoque.
     *
     * @param $filial
     * @param $nrorg
     * @param $nrLanc
     * @param $seq
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function excluiItem($filial, $nrorg, $nrLanc, $seq){
        try {
            /** @var $objItem Itlanctoest */
            $objItem = Itlanctoest::consultaItem($filial, $nrLanc, $nrorg, $seq);
            $cdProduto = $objItem->getProduto()->getCdproduto();
            $cdProdMovi = $objItem->getProdmovi()->getCdproduto();

            // Exclui Retirada de estoque associada.
            Ajuste::excluiRetiradas($nrorg, $filial, $nrLanc, $seq, $cdProduto, $cdProdMovi);

            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLanc);
//            $objItem->delete();
//            $connection->flush();

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            AtualizadorDeEstoque::atualizarEstoque($objLancto);

            $connection->flush();
            $connection->commit();

            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Verifica se ultimo item lancado.
     *
     * @param $cdfilial
     * @param $nrlanctoestoq
     * @param $nrorg
     *
     * @return bool
     */
    public static function validaExclusaoItensLancamento($cdfilial, $nrlanctoestoq, $nrorg){
        $itensLanc = Itlanctoest::findByCdfilialAndNrlancestqAndNrorg($cdfilial, $nrlanctoestoq, $nrorg);
        if (count($itensLanc) === 1) {
            ExceptionLancamentoEstoque::erroUltimoProdLanc();
        }

        return true;
    }


    /**
     * Insere novo item.
     *
     * @param $filial
     * @param $nrorg
     * @param $cdProduto
     * @param $qtProduto
     * @param $nrLanc
     * @param $dtLanc
     * @param $ordem
     * @param $seq
     * @param $vrBruto
     * @param $vrLiq
     * @param $lote
     * @param $subLote
     * @param $local
     * @param $cdAlmox
     * @param $idEvol
     * @param $tipoMovi
     * @param $vrConv
     * @param $operador
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function insereItemLancamento(
        $filial, $nrorg, $cdProduto, $qtProduto, $vrTotal, $nrLanc, $dtLanc, $ordem, $seq, $vrBruto, $vrLiq,
        $lote, $subLote, $local, $cdAlmox, $idEvol, $tipoMovi, $qtEstoq, $vrEst, $operador
    ){
        $dtLancto = DateUtil::getDataDeString($dtLanc, DateUtil::FORMATO_BRASILEIRO_DATAHORA);

        $item = new Itlanctoest();
        $item->insereItemLancto(
            $filial, $nrorg, $nrLanc, $dtLancto, $cdProduto, $local, $cdAlmox, $operador,
            $idEvol, $ordem, $tipoMovi, $qtEstoq, $vrEst, $qtProduto, $seq, $subLote, $lote, $vrBruto,
            $vrLiq, $vrTotal
        );

        return $item;
    }


    /**
     * Verifica se o produto tem controle de estoque.
     *
     * @param $cdProduto
     * @param $cdFilial
     *
     * @return bool
     */
    public static function produtoTemControleEstoque($cdProduto, $cdFilial) {
        /** @var $prodFili Prodfili */
        $prodFili = Prodfili::findOneByCdprodutoAndCdfilialAndNrorg($cdProduto, $cdFilial, Ambiente::getNrorg());
        if ($prodFili) {
            if ($prodFili->getIdcntrestoq() !== 'S') {
                ExceptionLancamentoEstoque::erroProdNaoCtrlEstoq();
            }
        }

        return true;
    }


    /**
     * Verifica arredondamento da quantidade lancada com a quantidade minima.
     *
     * @param $cdProduto
     * @param $filial
     * @param $nrorg
     * @param $qtdProduto
     *
     * @return bool
     */
    public static function validaArredondamento($cdProduto, $filial, $nrorg, $qtdProduto){
        /** @var $prodfili Prodfili */
        $prodfili = Prodfili::findOneByCdprodutoAndCdfilialAndNrorg($cdProduto, $filial, $nrorg);
        if ($prodfili) {
            $qtdMinSaida = $prodfili->getQtestminsaid();
            if (($qtdMinSaida > 0) && (empty($qtdMinSaida) !== true)) {
                if ($qtdProduto%$qtdMinSaida !== 0) {
                    ExceptionLancamentoEstoque::erroqtdMultiplo($qtdMinSaida);
                }
            }
        }

        return true;
    }




    /**
     * Exclui lancamento e seus itens.
     *
     * @param $nrorg
     * @param $nrLancto
     * @param $filial
     * @param string $tipoMovi
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function excluiLancamento($nrorg, $nrLancto, $filial, $tipoMovi = Itlanctoest::TIPO_AJUSTE){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            if ($tipoMovi === Itlanctoest::TIPO_AJUSTE) {
                Ajuste::excluiItemAjuste($nrorg, $filial, $nrLancto);
            } else {
                Implantacao::excluiItensImplantacao($nrorg, $filial, $nrLancto);
//                CalculaCustoLiquido::CalcularCustoLiquido($filial, $nrLancto, 0);
            }

            /** @var $objsCusto Custocmpprod */
            $objsCusto = Custocmpprod::getObjectsByWhere(
                'NRORG = ? AND CDFILLANCEST = ? AND NRLANCESTQ = ?',
                array($nrorg, $filial, $nrLancto)
            );

            if ($objsCusto) {
                /** @var $objCusto Custocmpprod */
                foreach ($objsCusto as $objCusto) {
                    $objCusto->delete();
                }

                $connection->flush();
            }

//            /** @var $objLancto Lanctoestoq */
//            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);
//            $objLancto->delete();

            $connection->flush();
            $connection->commit();

            return true;
        } catch(\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Insere novo item.
     *
     * @param $nrorg
     * @param $cdFilial
     * @param $cdOperador
     * @param $cdFiliMovi
     * @param $idTpLancto
     * @param $dtLancestq
     * @param $dsLancestq
     * @param null $nrlancestq
     *
     * @return \Estoque\Entities\Lanctoestoq
     *
     * @throws \Exception
     */
    public static function insereLancamentoEstoque(
        $nrorg, $cdFilial, $cdOperador, $cdFiliMovi, $idTpLancto, $dtLancestq, $dsLancestq, $nrlancestq = null, $cdalmoxarife = null, $cdlocalestoq = null
    ) {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            if (empty($nrlancestq) === true) {
                $nrlancestq = self::novoCodigo($nrorg, $cdFilial);
            }

            $lanctoestoq = new Lanctoestoq();
            $lanctoestoq->insere(
                $nrlancestq, $cdOperador, $cdFilial, $cdFiliMovi, $idTpLancto, $dtLancestq, $dsLancestq, $nrorg, $cdalmoxarife, $cdlocalestoq
            );

            $connection->flush();
            $connection->commit();

            return $lanctoestoq;
        } catch(\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Valida se o mes ja foi fechado.
     *
     * @param $dtProcessamento
     *
     * @return bool
     */
    public static function validaMesImplantacao($dtProcessamento){
        $diaFechamentoMes = Parametros::getDiaFechamentoMes();
        $diaBloq = Parametros::getDiaBloqueiaLancamento();

        $mesDtProc = DateUtil::getMes($dtProcessamento);
        $diaDtProc = DateUtil::getDia($dtProcessamento);

        $dtImpl = DateUtil::getDataAtual();
        $mesDtImpl = DateUtil::getMes($dtImpl);

        if ((empty($diaBloq) === false OR empty($diaFechamentoMes) === false) AND
            ($mesDtProc > $mesDtImpl OR $diaDtProc >= $diaFechamentoMes)
        ) {
            ExceptionLancamentoEstoque::erroMesFechado();
        }

        return true;
    }


    /**
     * Valida data de lancamento.
     *
     * @param $data
     * @param $origemParametro
     * @param $dtProcessamento
     *
     * @return bool
     */
    public static function validaData($data, $origemParametro, $dtProcessamento) {
        if (preg_match(DateUtil::DATE_TIME_PATTERN, $data)) {
            $data             = DateUtil::truncateString($data);
        }

        $data             = DateUtil::getDataDeString($data);
        $dtProcessamento  = DateUtil::getDataDeString($dtProcessamento);

        switch ($origemParametro) {
            case self::ENTRADA:
                $diasmaior = Parametros::getDiasEntradaMaior();
                $diasmenor = Parametros::getDiasEntradaMenor();
                break;
            case self::SAIDA:
                $diasmaior = Parametros::getDiasSaidaMaior();
                $diasmenor = Parametros::getDiasSaidaMenor();
                break;
            case self::AJUSTE:
                $diasmaior = Parametros::getDiasAjusteMaior();
                $diasmenor = Parametros::getDiasAjusteMenor();
                break;
            case self::IMPLANTACAO:
                $diasmaior = Parametros::getDiasAjusteMaior();
                $diasmenor = Parametros::getDiasAjusteMenor();
                break;
            default:
                return false;
        }

        self::verificaDataLancamentoIgual($data, $dtProcessamento);
        if (empty($diasmaior) !== true) {
            self::verificaDataLancamentoMaior($data, $dtProcessamento, $diasmaior);
        }

        if (empty($diasmenor) !== true) {
            self::verificaDataLancamentoMenor($data, $dtProcessamento, $diasmenor);
        }

        return true;
    }


    private static function verificaDataLancamentoIgual(\Setforms2\Shared\ORM\DateTime $data, \Setforms2\Shared\ORM\DateTime $dtprocessamento)
    {
        if (DateUtil::comparaDatas($data, $dtprocessamento, DateUtil::IGUAL)) {
            if (Parametros::getLancamentoDataMaior() !== 'S' AND Parametros::getLancamentoDataMenor() !== 'S') {
                ExceptionLancamentoEstoque::erroDtIgualDtProcessamento();
            }
        }

        return true;
    }


    private static function verificaDataLancamentoMaior(\Datetime $data, \DateTime $dtprocessamento, $diasmaior)
    {
        if (DateUtil::comparaDatas($data, $dtprocessamento, DateUtil::DEPOIS)) {
            if (Parametros::getLancamentoDataMaior() !== 'S') {
                ExceptionLancamentoEstoque::erroDtMaiorOuIgual();
            }

            $dtprocessdiasmaior = $dtprocessamento;
            $dtprocessdiasmaior = DateUtil::adicionaIntervalo($dtprocessdiasmaior, $diasmaior);

            if (DateUtil::comparaDatas($data, $dtprocessdiasmaior, DateUtil::DEPOIS)) {
                 ExceptionLancamentoEstoque::erroDtForaPeriodo();
            }
        }
        return true;
    }


    private static function verificaDataLancamentoMenor(\Datetime $data, \Datetime $dtprocessamento, $diasmenor)
    {
        if (DateUtil::comparaDatas($data, $dtprocessamento, DateUtil::ANTES)) {
            if (Parametros::getLancamentoDataMenor() !== 'S') {
                ExceptionLancamentoEstoque::erroDtMenorOuIgual();
            }

            $dtprocessdiasmenor = $dtprocessamento;
            $dtprocessdiasmenor = DateUtil::subtraiIntervalo($dtprocessdiasmenor, $diasmenor);
            if (DateUtil::comparaDatas($data, $dtprocessdiasmenor, DateUtil::ANTES)) {
                ExceptionLancamentoEstoque::erroDtForaPeriodo();
            }
        }

        return true;
    }


    /**
     * Busca o ultimo numero sequencial de um lancamento e retorna o mesmo + 1.
     *
     * @param $nrorg
     * @param $filial
     * @param $nrLancto
     *
     * @return string
     */
    public static function retornaMaxSequItem($nrorg, $filial, $nrLancto){
        /** @var $objItem Itlanctoest */
        $objItem = Itlanctoest::getObjectByWhere(
            'NRORG = ? AND NRLANCESTQ = ? AND CDFILIAL   = ?',
            array($nrorg, $nrLancto, $filial),
            'NRSEQUITEM DESC'
        );

        if ($objItem) {
            $nrSequitem = str_pad($objItem->getNrsequitem(), 4, "0", STR_PAD_LEFT);
        } else {
            $nrSequitem = str_pad(0, 4, "0", STR_PAD_LEFT);
        }

        return $nrSequitem;
    }


    /**
     * Verifica se a filial tem controle de estoque.
     *
     * @return bool
     */
    public static function verificaCtrlEstFilial(){
        $filialCtrlEstoque = \Estoque\Util\Parametros::getControlaEstoque();
        if ($filialCtrlEstoque !== 'S') {
            ExceptionLancamentoEstoque::erroFilialSemCtrlEst();
        }

        return true;
    }


    public static function produtoCadastrado($cdProduto, $nrLancestq, $cdFilial, $nrorg) {
        /** @var $itemNovo Produto */
        $itemNovo = Produto::getObject(array('nrorg' => $nrorg, 'cdfilial' => $cdFilial, 'cdproduto' => $cdProduto));
        $prodEstoNovo = $itemNovo->getCdprodesto();

        $itensLanc = Itlanctoest::consultaItens($cdFilial, $nrLancestq, $nrorg);
        if ($itensLanc){
            /** @var $item Itlanctoest */
            foreach ($itensLanc as $item) {
                $prodEsto = $item->getProdmovi()->getCdproduto();
                if ($prodEstoNovo === $prodEsto) {
//                    $params = $prodEsto." - ".$item->getProduto()->getNmproduto();
                    ExceptionLancamentoEstoque::erroProdutoCadastrado();
                }
            }
        }

        return true;
    }


    /**
     * Zera produtos em estoque.
     *
     * @param $cdFilial
     * @param $cdFilialItlancto
     * @param $dtLancamento
     * @param $nrLancamento
     * @param $nrorg
     * @param $origemParametro
     * @param $operador
     *
     * @return array
     *
     * @throws \Exception
     */
    public static function zeraEstoqueProdutos(
        $cdFilial, $cdFilialItlancto, \DateTime $dtLancamento, $nrLancamento, $nrorg, $origemParametro, $operador
    ) {
        try {
            /** @var $itens \Estoque\Repositories\Posestdia */
            $itens = Posestdia::getRepository();

            $listaLancto = null;
            $result = $itens->zeraEstoqueProdutos(
                $cdFilial, $cdFilialItlancto, $dtLancamento->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA),
                $nrLancamento, $nrorg
            );

            if ($result) {
                $qtdResult     = count($result);
                /** ceil - Arredonda sempre para cima. */
                $qtdLancto     = ceil($qtdResult / 9999);
                $restoOriginal = array_splice($result, 9999);

                /** @var $lanctoestoqOrig Lanctoestoq */
                $lanctoestoqOrig = Lanctoestoq::consultaLancamento($nrorg, $cdFilial, $nrLancamento);
                $listaLancto = array($lanctoestoqOrig);

                if ($origemParametro == Itlanctoest::TIPO_IMPLANTACAO){
                    if (Parametros::getOrdemImplantacao() == "P") {
                        $ordem = Itlanctoest::ORDEM_PRIMEIRO;
                    } else {
                        $ordem = Itlanctoest::ORDEM_ULTIMO;
                    }
                } else {
                    if (Parametros::getOrdemEvolucaoAjuste() == "P") {
                        $ordem = Itlanctoest::ORDEM_PRIMEIRO;
                    } else {
                        $ordem = Itlanctoest::ORDEM_PRIMEIRO;
                    }
                }

                $qtTotal = 1;
                while ($qtdLancto >= $qtTotal) {
                    /** @var $lanctoestoq Lanctoestoq */
                    $lanctoestoq = self::insereLancamentoEstoque(
                        $nrorg, $cdFilial, $lanctoestoqOrig->getCdoperinclusao(), $cdFilial,
                        $lanctoestoqOrig->getIdtplancto(), $dtLancamento,
                        'Zera estoque para o lançamento: '.$lanctoestoqOrig->getNrlancestq().' ('.$qtTotal.')'
                    );

                    $listaLancto[] = $lanctoestoq;
                    $nrSequitem = str_pad("0", 4, "0", STR_PAD_LEFT);

                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    $connection->beginTransaction();

                    foreach($result as $value) {
                        $nrSequitem = (float)$nrSequitem + 1;
                        $nrSequitem = str_pad($nrSequitem, 4, "0", STR_PAD_LEFT);

                        $itlanctoest = new Itlanctoest();
                        $itlanctoest->insereItemLancto(
                            $cdFilialItlancto, $nrorg, $lanctoestoq->getNrlancestq(), $dtLancamento, $value['CDPRODUTO'],
                            $value['CDLOCALESTOQ'], $value['CDALMOXARIFE'], $operador, Itlanctoest::ESTOQUE_NAO_EVOLUIDO,
                            $ordem, $lanctoestoq->getIdtplancto(), 0, 0, 0, $nrSequitem, $value['NRSUBLOTE'],
                            $value['NRLOTEESTQ'], 0, 0, 0
                        );
                        $connection->flush();

//                        if ((int)$nrSequitem === 0002){
                        if ((int)$nrSequitem === count($result)) {
                            break;
                        }
                    }

                    $connection->commit();

                    $result = $restoOriginal;
                    $restoOriginal = array_splice($result, 9999);
                    $qtTotal++;
                }
            }

//            $objMovi = Movilote::getObjectsByWhere('NRORG = ? AND NRMOVILOTE = ?',
//                array($nrorg, $cdFilial.$nrLancamento));
//
//            if ($objMovi) {
//                $connection = \Setforms2\Shared\Registry::getEntityManager();
//                $connection->beginTransaction();
//
//                /** @var $movi Movilote */
//                foreach ($objMovi as $movi) {
//                    $movi->delete();
//                    $connection->flush();
//                }
//
//                $connection->commit();
//            }

            return $listaLancto;
        }  catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Gera/retorna lote de acordo com a procedure.
     *
     * @param $filial
     * @param $nrorg
     * @param $subLote
     * @param $cdProduto
     *
     * @return string
     *
     * @throws \Exception
     */
    public static function geraLote($filial, $nrorg, $subLote, $cdProduto){
         try {
            $procedure = new \Setforms2\Runtime\Helper\Procedure('PRXLOTEPRODEST');

            $procedure->addParam(new Param("P_CDFILIAL",
                Param::PARAM_INPUT,
                $filial,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param("P_DTVALILOTE",
                Param::PARAM_INPUT,
                now(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param("P_CDPRODUTO",
                Param::PARAM_INPUT,
                $cdProduto,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param("P_RESULT",
                Param::PARAM_OUTPUT,
                null,
                Param::PARAM_TYPE_STR,
                20
            ));

            $lote = $procedure->call();
            $lote = $lote['P_RESULT'];

            if ($lote) {
                $loteExist = Lote::findOneByNrloteestqAndCdfilialAndNrorgAndNrsubloteAndCdproduto(
                    $lote, $filial, $nrorg, $subLote, $cdProduto
                );

                if(!$loteExist) {
                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    $connection->beginTransaction();

                    try {
                        $novoLote = new Lote();
                        $novoLote->insereLote($filial, $nrorg, $cdProduto, $lote, $subLote, Lote::LIBERADO);

                        $connection->flush();
                        $connection->commit();
                    } catch (\Exception $e) {
                        $connection->rollback();
                        throw $e;
                    }
                }
            }  else {
                $lote = \Geral\Entities\Lote::LOTEPADRAO;
            }

             return $lote;
         } catch (\Exception $e) {
            throw $e;
        }
    }


}